@@ -243,11 +243,15 @@ handle_screenshot_in_thread_func (GTask *task,
         {
           g_autoptr(GDesktopAppInfo) info = NULL;
           g_autofree gchar *id = NULL;
-          const gchar *name;
+          const gchar *name = NULL;
 
           id = g_strconcat (app_id, ".desktop", NULL);
           info = g_desktop_app_info_new (id);
-          name = g_app_info_get_display_name (G_APP_INFO (info));
+
+          if (info)
+            name = g_app_info_get_display_name (G_APP_INFO (info));
+          else
+            name = app_id;
 
           title = g_strdup_printf (_("Allow %s to Take Screenshots?"), name);
           subtitle = g_strdup_printf (_("%s wants to be able to take screenshots at any time."), name);
@@ -278,6 +282,7 @@ handle_screenshot_in_thread_func (GTask *task,
                                                          &error))
         {
           g_warning ("Failed to show access dialog: %s", error->message);
+          send_response (request, 2, g_variant_builder_end (&opt_builder));
           return;
         }
 
