@@ -183,7 +183,7 @@ _xdp_parse_app_id_from_unit_name (const char *unit)
 
   if (match != NULL)
     {
-      const char *escaped_app_id;
+      g_autofree char *escaped_app_id = NULL;
       /* Unescape the unit name which may have \x hex codes in it, e.g.
        * "app-gnome-org.gnome.Evolution\x2dalarm\x2dnotify-2437.scope"
        */
@@ -397,8 +397,10 @@ xdp_app_info_rewrite_commandline (XdpAppInfo *app_info,
           for (i = 1; commandline[i]; i++)
             g_ptr_array_add (args, maybe_quote (commandline[i], quote_escape));
         }
-      else
+      else if (quote_escape)
         g_ptr_array_add (args, g_shell_quote (app_info->id));
+      else
+        g_ptr_array_add (args, g_strdup (app_info->id));
       g_ptr_array_add (args, NULL);
 
       return (char **)g_ptr_array_free (g_steal_pointer (&args), FALSE);
@@ -1054,6 +1056,8 @@ xdg_desktop_portal_error_quark (void)
   return (GQuark) quark_volatile;
 }
 
+static char *documents_mountpoint = NULL;
+
 static char *
 verify_proc_self_fd (XdpAppInfo *app_info,
                      const char *proc_path,
@@ -1092,17 +1096,28 @@ verify_proc_self_fd (XdpAppInfo *app_info,
      but there is not much to do about this. */
   if (g_str_has_suffix (path_buffer, " (deleted)"))
     {
-      g_set_error (error, G_IO_ERROR, G_IO_ERROR_INVALID_FILENAME,
-                   "Cannot share deleted file: %s", path_buffer);
-      return NULL;
+      if (documents_mountpoint != NULL &&
+          g_str_has_prefix (path_buffer, documents_mountpoint))
+        {
+          /* Unfortunately our workaround for dcache purging triggers
+             o_path file descriptors on the fuse filesystem being
+             marked as deleted, so we have to allow these here and
+             rewrite them. This is safe, becase we will stat the file
+             and compare to make sure we end up on the right file. */
+          path_buffer[symlink_size - strlen(" (deleted)")] = 0;
+        }
+      else
+        {
+          g_set_error (error, G_IO_ERROR, G_IO_ERROR_INVALID_FILENAME,
+                       "Cannot share deleted file: %s", path_buffer);
+          return NULL;
+        }
     }
 
   /* remap from sandbox to host if needed */
   return xdp_app_info_remap_path (app_info, path_buffer);
 }
 
-static char *documents_mountpoint = NULL;
-
 void
 xdp_set_documents_mountpoint (const char *path)
 {
@@ -1403,6 +1418,15 @@ xdp_is_valid_app_id (const char *string)
   return TRUE;
 }
 
+char *
+xdp_get_app_id_from_desktop_id (const char *desktop_id)
+{
+  const gchar *suffix = ".desktop";
+  if (g_str_has_suffix (desktop_id, suffix))
+    return g_strndup (desktop_id, strlen (desktop_id) - strlen (suffix));
+  else
+    return g_strdup (desktop_id);
+}
 
 char *
 xdp_quote_argv (const char *argv[])
@@ -2165,6 +2189,9 @@ app_info_map_pids (XdpAppInfo  *app_info,
   g_return_val_if_fail (app_info != NULL, FALSE);
   g_return_val_if_fail (pids != NULL, FALSE);
 
+  if (app_info->kind == XDP_APP_INFO_KIND_HOST)
+    return TRUE;
+
   if (app_info->kind != XDP_APP_INFO_KIND_FLATPAK)
     {
       g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
